@@ -89,9 +89,6 @@ jbig2_detect_language(const char *s, int len)
 static const pl_interp_characteristics_t jbig2_characteristics = {
     "JBIG2",
     jbig2_detect_language,
-    "Artifex",
-    "0.01",
-    "18 Nov 2019"
 };
 
 /* Get implementation's characteristics */
@@ -608,5 +605,6 @@ const pl_interp_implementation_t jbig2_implementation = {
   jbig2_impl_report_errors,
   jbig2_impl_dnit_job,
   jbig2_impl_deallocate_interp_instance,
-  NULL
+  NULL, /* jbig2_impl_reset */
+  NULL  /* interp_client_data */
 };
