@@ -113,9 +113,6 @@ jp2k_detect_language(const char *s_, int len)
 static const pl_interp_characteristics_t jp2k_characteristics = {
     "JP2K",
     jp2k_detect_language,
-    "Artifex",
-    "0.01",
-    "22 Nov 2019"
 };
 
 /* Get implementation's characteristics */
@@ -721,5 +718,6 @@ const pl_interp_implementation_t jp2k_implementation = {
   jp2k_impl_report_errors,
   jp2k_impl_dnit_job,
   jp2k_impl_deallocate_interp_instance,
-  NULL
+  NULL, /* jp2k_impl_reset */
+  NULL  /* interp_client_data */
 };
