@@ -118,9 +118,6 @@ jpg_detect_language(const char *s, int len)
 static const pl_interp_characteristics_t jpg_characteristics = {
     "JPG",
     jpg_detect_language,
-    "Artifex",
-    "0.01",
-    "11 Nov 2019"
 };
 
 /* GS's fakakta jpeg integration insists on putting a
@@ -870,5 +867,6 @@ const pl_interp_implementation_t jpg_implementation = {
   jpg_impl_report_errors,
   jpg_impl_dnit_job,
   jpg_impl_deallocate_interp_instance,
-  NULL
+  NULL, /* jpg_impl_reset */
+  NULL  /* interp_client_data */
 };
