@@ -110,9 +110,6 @@ png_detect_language(const char *s, int len)
 static const pl_interp_characteristics_t png_characteristics = {
     "PNG",
     png_detect_language,
-    "Artifex",
-    "0.01",
-    "11 Nov 2019"
 };
 
 /* Get implementation's characteristics */
@@ -795,5 +792,6 @@ const pl_interp_implementation_t png_implementation = {
   png_impl_report_errors,
   png_impl_dnit_job,
   png_impl_deallocate_interp_instance,
-  NULL
+  NULL, /* png_impl_reset */
+  NULL  /* interp_client_data */
 };
