@@ -302,17 +302,10 @@ static const pl_interp_characteristics_t * /* always returns a descriptor */
 ps_impl_characteristics(const pl_interp_implementation_t *impl)     /* implementation of interpreter to alloc */
 {
     /* version and build date are not currently used */
-#define PSVERSION NULL
-#define PSBUILDDATE NULL
   static const pl_interp_characteristics_t ps_characteristics = {
     "POSTSCRIPT",
     ps_detect_language,
-    "Artifex",
-    PSVERSION,
-    PSBUILDDATE
   };
-#undef PSVERSION
-#undef PSBUILDDATE
 
   return &ps_characteristics;
 }
@@ -663,5 +656,6 @@ const pl_interp_implementation_t ps_implementation = {
   ps_impl_report_errors,
   ps_impl_dnit_job,
   ps_impl_deallocate_interp_instance,
-  NULL
+  NULL, /* ps_impl_reset */
+  NULL  /* interp_client_data */
 };
