@@ -103,9 +103,6 @@ pwg_detect_language(const char *s, int len)
 static const pl_interp_characteristics_t pwg_characteristics = {
     "PWG",
     pwg_detect_language,
-    "Artifex",
-    "0.01",
-    "13 Nov 2019"
 };
 
 /* Get implementation's characteristics */
@@ -717,5 +714,6 @@ const pl_interp_implementation_t pwg_implementation = {
   pwg_impl_report_errors,
   pwg_impl_dnit_job,
   pwg_impl_deallocate_interp_instance,
-  NULL
+  NULL, /* pwg_impl_reset */
+  NULL  /* interp_client_data */
 };
