@@ -114,9 +114,6 @@ tiff_detect_language(const char *s, int len)
 static const pl_interp_characteristics_t tiff_characteristics = {
     "TIFF",
     tiff_detect_language,
-    "Artifex",
-    "0.01",
-    "18 Nov 2019"
 };
 
 /* Get implementation's characteristics */
@@ -1123,5 +1120,6 @@ const pl_interp_implementation_t tiff_implementation = {
   tiff_impl_report_errors,
   tiff_impl_dnit_job,
   tiff_impl_deallocate_interp_instance,
-  NULL
+  NULL, /* tiff_impl_reset */
+  NULL  /* interp_client_data */
 };
