@@ -225,11 +225,11 @@ pcl_do_resets(pcl_state_t * pcs, pcl_reset_type_t type)
     }
 
     for (; *init && code >= 0; ++init) {
-        if ((*init)->do_reset)
+        if ((*init)->do_reset) {
             code = (*(*init)->do_reset) (pcs, type);
-    }
-    if (code < 0) {
-        (void) pcl_do_resets_permanent(pcs);
+            if (code < 0)
+                break;
+        }
     }
     return code;
 }
