@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -41,6 +41,7 @@
 #include "gxpath.h"
 #include "gzstate.h"
 #include "pxptable.h"
+#include "pxpthr.h"
 
 #include "plfapi.h"
 
@@ -334,6 +335,7 @@ px_text_setup(gs_gstate * pgs, const gs_char * str, uint size,
         text.operation |= TEXT_DO_TRUE_CHARPATH;
     else
         text.operation |= TEXT_DO_DRAW;
+    text.operation |= can_cache ? 0 : TEXT_NO_CACHE;
     text.data.chars = str;
     text.size = size;
     text.x_widths = x_widths;
@@ -343,12 +345,6 @@ px_text_setup(gs_gstate * pgs, const gs_char * str, uint size,
     if (code < 0)
         return code;
 
-    if (!can_cache) {
-        /* NB breaks API, needs a better solution. */
-        gs_show_enum *penum = (gs_show_enum *) * ppte;
-
-        penum->can_cache = -1;
-    }
     return code;
 }
 
@@ -463,7 +459,7 @@ px_text(px_args_t * par, px_state_t * pxs, bool to_path)
 
         if (code >= 0) {
             code = gs_text_process(penum);
-            gs_text_release(penum, "pxtext");
+            gs_text_release(pgs, penum, "pxtext");
         }
         if (fxvals)
             gs_free_object(mem, fxvals, "px_text fvals");
@@ -534,9 +530,6 @@ pxSetFont(px_args_t * par, px_state_t * pxs)
         px_set_symbol_map(pxs, pxfont->font_type == plft_16bit);
         pxgs->char_matrix_set = false;
     } else {                    /* PCLSelectFont */
-
-        extern int pxpcl_selectfont(px_args_t * par, px_state_t * pxs);
-
         code = pxpcl_selectfont(par, pxs);
         if (code < 0)
             return code;
@@ -726,10 +719,7 @@ pxReadChar(px_args_t * par, px_state_t * pxs)
                     code = gs_note_error(errorFSTMismatch);
                 else if (data[1] != 0)
                     code = gs_note_error(errorUnsupportedCharacterClass);
-                else if (size < 10 ||
-                         size != 10 + ((pl_get_uint16(data + 6) + 7) >> 3) *
-                         pl_get_uint16(data + 8)
-                    )
+                else if (size < 10)
                     code = gs_note_error(errorIllegalCharacterData);
                 else {
                     int loff = pl_get_int16(data + 2);
@@ -739,7 +729,7 @@ pxReadChar(px_args_t * par, px_state_t * pxs)
                     uint bmp_size = ((width + 7) >> 3) * height;
                     uint bmp_offset = round_up(10, ARCH_ALIGN_PTR_MOD);
 
-                    if (size < 10 || size != 10 + ((width + 7) >> 3) * height)
+                    if (size != 10 + ((width + 7) >> 3) * height)
                         code = gs_note_error(errorIllegalCharacterData);
                     else if ((-16384 > toff) || (toff > 16384))
                         code = gs_note_error(errorIllegalCharacterData);
@@ -838,7 +828,7 @@ pxRemoveFont(px_args_t * par, px_state_t * pxs)
          */
         if (pxfont->pfont == gs_currentfont(pxs->pgs))
             gs_setfont(pxs->pgs, NULL);
-        
+
         px_dict_undef(&pxs->font_dict, par->pv[0]);
     }
 
