@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -18,37 +18,42 @@
 
 #include "ghostxps.h"
 
-static int isfile(char *path)
+static int isfile(gs_memory_t *mem, char *path)
 {
-    FILE *file = gp_fopen(path, "rb");
+    gp_file *file = gp_fopen(mem, path, "rb");
     if (file)
     {
-        fclose(file);
+        gp_fclose(file);
         return 1;
     }
     return 0;
 }
 
-static inline int getshort(FILE *file)
+static inline int getshort(gp_file *file)
 {
-    int a = getc(file);
-    int b = getc(file);
+    int a = xps_getc(file);
+    int b = xps_getc(file);
     return a | (b << 8);
 }
 
-static inline int getlong(FILE *file)
+static inline int getlong(gp_file *file)
 {
-    int a = getc(file);
-    int b = getc(file);
-    int c = getc(file);
-    int d = getc(file);
+    int a = xps_getc(file);
+    int b = xps_getc(file);
+    int c = xps_getc(file);
+    int d = xps_getc(file);
     return a | (b << 8) | (c << 16) | (d << 24);
 }
 
 static void *
 xps_zip_alloc_items(xps_context_t *ctx, int items, int size)
 {
-    return xps_alloc(ctx, items * size);
+    void *item = xps_alloc(ctx, (size_t)items * size);
+    if (!item) {
+        gs_throw(gs_error_VMerror, "out of memory: item.\n");
+        return NULL;
+    }
+    return item;
 }
 
 static void
@@ -66,7 +71,7 @@ xps_compare_entries(const void *a0, const void *b0)
 }
 
 static xps_entry_t *
-xps_find_zip_entry(xps_context_t *ctx, char *name)
+xps_find_zip_entry(xps_context_t *ctx, const char *name)
 {
     int l = 0;
     int r = ctx->zip_count - 1;
@@ -100,14 +105,18 @@ xps_read_zip_entry(xps_context_t *ctx, xps_entry_t *ent, unsigned char *outbuf)
 
     if_debug1m('|', ctx->memory, "zip: inflating entry '%s'\n", ent->name);
 
-    fseek(ctx->file, ent->offset, 0);
+    if (xps_fseek(ctx->file, ent->offset, 0) < 0)
+        return gs_throw1(-1, "seek to offset %d failed.", ent->offset);
 
     sig = getlong(ctx->file);
     if (sig != ZIP_LOCAL_FILE_SIG)
         return gs_throw1(-1, "wrong zip local file signature (0x%x)", sig);
 
     version = getshort(ctx->file);
+    (void)version; /* Avoid unused variable compiler warning */
     general = getshort(ctx->file);
+    if (general & ZIP_ENCRYPTED_FLAG)
+        return gs_throw(-1, "zip file content is encrypted");
     method = getshort(ctx->file);
     (void) getshort(ctx->file); /* file time */
     (void) getshort(ctx->file); /* file date */
@@ -117,17 +126,32 @@ xps_read_zip_entry(xps_context_t *ctx, xps_entry_t *ent, unsigned char *outbuf)
     namelength = getshort(ctx->file);
     extralength = getshort(ctx->file);
 
-    fseek(ctx->file, namelength + extralength, 1);
+    if (namelength < 0 || namelength > 65535)
+        return gs_rethrow(gs_error_ioerror, "Illegal namelength (can't happen).\n");
+    if (extralength < 0 || extralength > 65535)
+        return gs_rethrow(gs_error_ioerror, "Illegal extralength (can't happen).\n");
+
+    if (xps_fseek(ctx->file, namelength + extralength, 1) != 0)
+        return gs_throw1(gs_error_ioerror, "xps_fseek to %d failed.\n", namelength + extralength);
 
     if (method == 0)
     {
-        fread(outbuf, 1, ent->usize, ctx->file);
+        code = xps_fread(outbuf, 1, ent->usize, ctx->file);
+        if (code != ent->usize)
+            return gs_throw1(gs_error_ioerror, "Failed to read %d bytes", ent->usize);
     }
     else if (method == 8)
     {
         inbuf = xps_alloc(ctx, ent->csize);
+        if (!inbuf) {
+            return gs_rethrow(gs_error_VMerror, "out of memory.\n");
+        }
 
-        fread(inbuf, 1, ent->csize, ctx->file);
+        code = xps_fread(inbuf, 1, ent->csize, ctx->file);
+        if (code != ent->csize) {
+            xps_free(ctx, inbuf);
+            return gs_throw1(gs_error_ioerror, "Failed to read %d bytes", ent->csize);
+        }
 
         memset(&stream, 0, sizeof(z_stream));
         stream.zalloc = (alloc_func) xps_zip_alloc_items;
@@ -159,6 +183,13 @@ xps_read_zip_entry(xps_context_t *ctx, xps_entry_t *ent, unsigned char *outbuf)
         }
 
         xps_free(ctx, inbuf);
+
+        /* If the stream has less data than advertised, then zero the remainder. */
+        if (stream.avail_out > 0)
+        {
+            gs_warn("truncated zipfile entry; possibly corrupt data");
+            memset(stream.next_out, 0, stream.avail_out);
+        }
     }
     else
     {
@@ -176,11 +207,12 @@ static int
 xps_read_zip_dir(xps_context_t *ctx, int start_offset)
 {
     int sig;
-    int offset, count;
+    int offset, count, read;
     int namesize, metasize, commentsize;
     int i;
 
-    fseek(ctx->file, start_offset, 0);
+    if (xps_fseek(ctx->file, start_offset, 0) != 0)
+        return gs_throw1(gs_error_ioerror, "xps_fseek to %d failed.", start_offset);
 
     sig = getlong(ctx->file);
     if (sig != ZIP_END_OF_CENTRAL_DIRECTORY_SIG)
@@ -193,14 +225,18 @@ xps_read_zip_dir(xps_context_t *ctx, int start_offset)
     (void) getlong(ctx->file); /* size of central directory */
     offset = getlong(ctx->file); /* offset to central directory */
 
+    if (count < 0 || count > 65535)
+        return gs_rethrow(gs_error_rangecheck, "invalid number of entries in central directory disk (can't happen)");
+
     ctx->zip_count = count;
     ctx->zip_table = xps_alloc(ctx, sizeof(xps_entry_t) * count);
     if (!ctx->zip_table)
-        return gs_throw(-1, "cannot allocate zip entry table");
+        return gs_rethrow(gs_error_VMerror, "cannot allocate zip entry table");
 
     memset(ctx->zip_table, 0, sizeof(xps_entry_t) * count);
 
-    fseek(ctx->file, offset, 0);
+    if (xps_fseek(ctx->file, offset, 0) != 0)
+        return gs_throw1(gs_error_ioerror, "xps_fseek to offset %d failed", offset);
 
     for (i = 0; i < count; i++)
     {
@@ -218,6 +254,8 @@ xps_read_zip_dir(xps_context_t *ctx, int start_offset)
         ctx->zip_table[i].csize = getlong(ctx->file);
         ctx->zip_table[i].usize = getlong(ctx->file);
         namesize = getshort(ctx->file);
+        if (namesize < 0 || namesize > 65535)
+            return gs_rethrow(gs_error_rangecheck, "illegal namesize (can't happen)");
         metasize = getshort(ctx->file);
         commentsize = getshort(ctx->file);
         (void) getshort(ctx->file); /* disk number start */
@@ -225,15 +263,23 @@ xps_read_zip_dir(xps_context_t *ctx, int start_offset)
         (void) getlong(ctx->file); /* ext file atts */
         ctx->zip_table[i].offset = getlong(ctx->file);
 
+	if (ctx->zip_table[i].csize < 0 || ctx->zip_table[i].usize < 0)
+            return gs_throw(gs_error_ioerror, "cannot read zip entries larger than 2GB");
+
         ctx->zip_table[i].name = xps_alloc(ctx, namesize + 1);
         if (!ctx->zip_table[i].name)
-            return gs_throw(-1, "cannot allocate zip entry name");
+            return gs_rethrow(gs_error_VMerror, "cannot allocate zip entry name");
+
+        read = xps_fread(ctx->zip_table[i].name, 1, namesize, ctx->file);
+        if (read != namesize)
+            return gs_throw1(gs_error_ioerror, "failed to read %d bytes", namesize);
 
-        fread(ctx->zip_table[i].name, 1, namesize, ctx->file);
         ctx->zip_table[i].name[namesize] = 0;
 
-        fseek(ctx->file, metasize, 1);
-        fseek(ctx->file, commentsize, 1);
+        if (xps_fseek(ctx->file, metasize, 1) != 0)
+            return gs_throw1(gs_error_ioerror, "xps_fseek to offset %d failed", metasize);
+        if (xps_fseek(ctx->file, commentsize, 1) != 0)
+            return gs_throw1(gs_error_ioerror, "xps_fseek to offset %d failed", commentsize);
     }
 
     qsort(ctx->zip_table, count, sizeof(xps_entry_t), xps_compare_entries);
@@ -256,17 +302,20 @@ xps_find_and_read_zip_dir(xps_context_t *ctx)
     int i, n;
     char buf[512];
 
-    fseek(ctx->file, 0, SEEK_END);
-    filesize = ftell(ctx->file);
+    if (xps_fseek(ctx->file, 0, SEEK_END) < 0)
+        return gs_throw(-1, "seek to end failed.");
+
+    filesize = xps_ftell(ctx->file);
 
     maxback = MIN(filesize, 0xFFFF + sizeof buf);
     back = MIN(maxback, sizeof buf);
 
     while (back < maxback)
     {
-        fseek(ctx->file, filesize - back, 0);
+        if (xps_fseek(ctx->file, filesize - back, 0) < 0)
+            return gs_throw1(gs_error_ioerror, "xps_fseek to %d failed.\n", filesize - back);
 
-        n = fread(buf, 1, sizeof buf, ctx->file);
+        n = xps_fread(buf, 1, sizeof buf, ctx->file);
         if (n < 0)
             return gs_throw(-1, "cannot read end of central directory");
 
@@ -285,13 +334,15 @@ xps_find_and_read_zip_dir(xps_context_t *ctx)
  */
 
 static xps_part_t *
-xps_read_zip_part(xps_context_t *ctx, char *partname)
+xps_read_zip_part(xps_context_t *ctx, const char *partname)
 {
     char buf[2048];
     xps_entry_t *ent;
     xps_part_t *part;
     int count, size, offset, i;
-    char *name;
+    int code = 0;
+    const char *name;
+    int seen_last = 0;
 
     name = partname;
     if (name[0] == '/')
@@ -302,14 +353,23 @@ xps_read_zip_part(xps_context_t *ctx, char *partname)
     if (ent)
     {
         part = xps_new_part(ctx, partname, ent->usize);
-        xps_read_zip_entry(ctx, ent, part->data);
+        if (part != NULL)
+            code = xps_read_zip_entry(ctx, ent, part->data);
+        else
+            return NULL;
+        if (code < 0)
+        {
+            xps_free_part(ctx, part);
+            gs_rethrow1(-1, "cannot read zip entry '%s'", name);
+            return NULL;
+        }
         return part;
     }
 
     /* Count the number of pieces and their total size */
     count = 0;
     size = 0;
-    while (1)
+    while (!seen_last)
     {
         gs_sprintf(buf, "%s/[%d].piece", name, count);
         ent = xps_find_zip_entry(ctx, buf);
@@ -317,12 +377,18 @@ xps_read_zip_part(xps_context_t *ctx, char *partname)
         {
             gs_sprintf(buf, "%s/[%d].last.piece", name, count);
             ent = xps_find_zip_entry(ctx, buf);
+            seen_last = !!ent;
         }
         if (!ent)
             break;
         count ++;
         size += ent->usize;
     }
+    if (!seen_last)
+    {
+        gs_throw1(-1, "cannot find all pieces for part '%s'", partname);
+        return NULL;
+    }
 
     /* Inflate the pieces */
     if (count)
@@ -340,7 +406,13 @@ xps_read_zip_part(xps_context_t *ctx, char *partname)
                 gs_warn("missing piece");
             else
             {
-                xps_read_zip_entry(ctx, ent, part->data + offset);
+                code = xps_read_zip_entry(ctx, ent, part->data + offset);
+                if (code < 0)
+                {
+                    xps_free_part(ctx, part);
+                    gs_rethrow1(-1, "cannot read zip entry '%s'", buf);
+                    return NULL;
+                }
                 offset += ent->usize;
             }
         }
@@ -355,27 +427,43 @@ xps_read_zip_part(xps_context_t *ctx, char *partname)
  */
 
 static xps_part_t *
-xps_read_dir_part(xps_context_t *ctx, char *name)
+xps_read_dir_part(xps_context_t *ctx, const char *name)
 {
     char buf[2048];
     xps_part_t *part;
-    FILE *file;
+    gp_file *file;
     int count, size, offset, i, n;
 
-    xps_strlcpy(buf, ctx->directory, sizeof buf);
-    xps_strlcat(buf, name, sizeof buf);
+    gs_strlcpy(buf, ctx->directory, sizeof buf);
+    gs_strlcat(buf, name, sizeof buf);
 
     /* All in one piece */
-    file = gp_fopen(buf, "rb");
+    file = gp_fopen(ctx->memory, buf, "rb");
     if (file)
     {
-        fseek(file, 0, SEEK_END);
-        size = ftell(file);
-        fseek(file, 0, SEEK_SET);
+        if (xps_fseek(file, 0, SEEK_END) != 0) {
+            gp_fclose(file);
+            return NULL;
+        }
+
+        size = xps_ftell(file);
+        if (size < 0) {
+            gp_fclose(file);
+            return NULL;
+        }
+
+        if (xps_fseek(file, 0, SEEK_SET) != 0) {
+            gp_fclose(file);
+            return NULL;
+        }
+
         part = xps_new_part(ctx, name, size);
-        fread(part->data, 1, size, file);
-        fclose(file);
-        return part;
+        count = xps_fread(part->data, 1, size, file);
+        gp_fclose(file);
+        if (count == size)
+            return part;
+        else
+            return NULL;
     }
 
     /* Count the number of pieces and their total size */
@@ -384,18 +472,19 @@ xps_read_dir_part(xps_context_t *ctx, char *name)
     while (1)
     {
         gs_sprintf(buf, "%s%s/[%d].piece", ctx->directory, name, count);
-        file = gp_fopen(buf, "rb");
+        file = gp_fopen(ctx->memory, buf, "rb");
         if (!file)
         {
             gs_sprintf(buf, "%s%s/[%d].last.piece", ctx->directory, name, count);
-            file = gp_fopen(buf, "rb");
+            file = gp_fopen(ctx->memory, buf, "rb");
         }
         if (!file)
             break;
         count ++;
-        fseek(file, 0, SEEK_END);
-        size += ftell(file);
-        fclose(file);
+        if (xps_fseek(file, 0, SEEK_END) < 0)
+            break;;
+        size += xps_ftell(file);
+        gp_fclose(file);
     }
 
     /* Inflate the pieces */
@@ -409,10 +498,10 @@ xps_read_dir_part(xps_context_t *ctx, char *name)
                 gs_sprintf(buf, "%s%s/[%d].piece", ctx->directory, name, i);
             else
                 gs_sprintf(buf, "%s%s/[%d].last.piece", ctx->directory, name, i);
-            file = gp_fopen(buf, "rb");
-            n = fread(part->data + offset, 1, size - offset, file);
+            file = gp_fopen(ctx->memory, buf, "rb");
+            n = xps_fread(part->data + offset, 1, size - offset, file);
             offset += n;
-            fclose(file);
+            gp_fclose(file);
         }
         return part;
     }
@@ -421,7 +510,7 @@ xps_read_dir_part(xps_context_t *ctx, char *name)
 }
 
 xps_part_t *
-xps_read_part(xps_context_t *ctx, char *partname)
+xps_read_part(xps_context_t *ctx, const char *partname)
 {
     if (ctx->directory)
         return xps_read_dir_part(ctx, partname);
@@ -433,7 +522,7 @@ xps_read_part(xps_context_t *ctx, char *partname)
  */
 
 static int
-xps_read_and_process_metadata_part(xps_context_t *ctx, char *name)
+xps_read_and_process_metadata_part(xps_context_t *ctx, const char *name)
 {
     xps_part_t *part;
     int code;
@@ -476,12 +565,206 @@ xps_read_and_process_page_part(xps_context_t *ctx, char *name)
     return gs_okay;
 }
 
+/* XPS page reordering based upon Device PageList setting */
+static int
+xps_reorder_add_page(xps_context_t* ctx, xps_page_t ***page_ptr, xps_page_t* page_to_add)
+{
+    xps_page_t* new_page;
+
+    new_page = xps_alloc(ctx, sizeof(xps_page_t));
+    if (!new_page)
+    {
+        return gs_throw(gs_error_VMerror, "out of memory: xps_reorder_add_page\n");
+    }
+
+    new_page->name = xps_strdup(ctx, page_to_add->name);
+    if (!new_page->name)
+    {
+        return gs_throw(gs_error_VMerror, "out of memory: xps_reorder_add_page\n");
+    }
+    new_page->height = page_to_add->height;
+    new_page->width = page_to_add->width;
+    new_page->next = NULL;
+
+    **page_ptr = new_page;
+    *page_ptr = &(new_page->next);
+
+    return 0;
+}
+
+static char*
+xps_reorder_get_range(xps_context_t *ctx, char *page_list, int *start, int *end, int num_pages)
+{
+    int comma, dash, len;
+
+    len = strlen(page_list);
+    comma = strcspn(page_list, ",");
+    dash = strcspn(page_list, "-");
+
+    if (dash < comma)
+    {
+        /* Dash at start */
+        if (dash == 0)
+        {
+            *start = num_pages;
+            *end = atoi(&(page_list[dash + 1]));
+        }
+        else
+        {
+            *start = atoi(page_list);
+
+            /* Dash at end */
+            if (page_list[dash + 1] == 0 || page_list[dash + 1] == ',')
+            {
+                *end = num_pages;
+            }
+            else
+            {
+                *end = atoi(&(page_list[dash + 1]));
+            }
+        }
+    }
+    else
+    {
+        *start = atoi(page_list);
+        *end = *start;
+    }
+    return comma == len ? page_list + comma : page_list + comma + 1;
+}
+
+static int
+xps_reorder_pages(xps_context_t *ctx)
+{
+    char *page_list = ctx->page_range->page_list;
+    char *str;
+    xps_page_t **page_ptr_array, *page = ctx->first_page;
+    int count = 0, k;
+    int code;
+    int start;
+    int end;
+    xps_page_t* first_page = NULL;
+    xps_page_t* last_page;
+    xps_page_t** page_tail = &first_page;
+
+    if (page == NULL)
+        return 0;
+
+    while (page != NULL)
+    {
+        count++;
+        page = page->next;
+    }
+
+    /* Create an array of pointers to the current pages */
+    page_ptr_array = xps_alloc(ctx, sizeof(xps_page_t*) * count);
+    if (page_ptr_array == NULL)
+        return gs_throw(gs_error_VMerror, "out of memory: xps_reorder_pages\n");
+
+    page = ctx->first_page;
+    for (k = 0; k < count; k++)
+    {
+        page_ptr_array[k] = page;
+        page = page->next;
+    }
+
+    if (strcmp(page_list, "even") == 0)
+    {
+        for (k = 1; k < count; k += 2)
+        {
+            code = xps_reorder_add_page(ctx, &page_tail, page_ptr_array[k]);
+            if (code < 0)
+                return code;
+        }
+    }
+    else if (strcmp(page_list, "odd") == 0)
+    {
+        for (k = 0; k < count; k += 2)
+        {
+            code = xps_reorder_add_page(ctx, &page_tail, page_ptr_array[k]);
+            if (code < 0)
+                return code;
+        }
+    }
+    else
+    {
+        /* Requirements. All characters must be 0 to 9 or - and ,
+          No ,, or --  */
+        str = page_list;
+        do
+        {
+            if (*str != ',' && *str != '-' && (*str < 0x30 || *str > 0x39))
+                return gs_throw(gs_error_typecheck, "Bad page list: xps_reorder_pages\n");
+
+            if ((*str == ',' && *(str + 1) == ',') || (*str == '-' && *(str + 1) == '-'))
+                return gs_throw(gs_error_typecheck, "Bad page list: xps_reorder_pages\n");
+        }
+        while (*(++str));
+
+        str = page_list;
+        do
+        {
+            /* Process each comma separated item. */
+            str = xps_reorder_get_range(ctx, str, &start, &end, count);
+
+            /* Threshold page range */
+            if (start > count)
+                start = count;
+
+            if (end > count)
+                end = count;
+
+            /* Add page(s) */
+            if (start == end)
+            {
+                code = xps_reorder_add_page(ctx, &page_tail, page_ptr_array[start - 1]);
+                if (code < 0)
+                    return code;
+            }
+            else if (start < end)
+            {
+                for (k = start - 1; k < end; k++)
+                {
+                    code = xps_reorder_add_page(ctx, &page_tail, page_ptr_array[k]);
+                    if (code < 0)
+                        return code;
+                }
+            }
+            else
+            {
+                for (k = start; k >= end; k--)
+                {
+                    code = xps_reorder_add_page(ctx, &page_tail, page_ptr_array[k - 1]);
+                    if (code < 0)
+                        return code;
+                }
+            }
+        }
+        while (*str);
+    }
+
+    /* Replace the pages. */
+    if (first_page != NULL)
+    {
+        /* Set to the last page not its next pointer (Thanks to Robin Watts) */
+        last_page = (xps_page_t*)(((char*)page_tail) - offsetof(xps_page_t, next));
+
+        xps_free_fixed_pages(ctx);
+        xps_free(ctx, page_ptr_array);
+        ctx->first_page = first_page;
+        ctx->last_page = last_page;
+    }
+    else
+        return gs_throw(gs_error_rangecheck, "Bad page list: xps_reorder_pages\n");
+
+    return 0;
+}
+
 /*
  * Called by xpstop.c
  */
 
 int
-xps_process_file(xps_context_t *ctx, char *filename)
+xps_process_file(xps_context_t *ctx, const char *filename)
 {
     char buf[2048];
     xps_document_t *doc;
@@ -489,7 +772,7 @@ xps_process_file(xps_context_t *ctx, char *filename)
     int code;
     char *p;
 
-    ctx->file = gp_fopen(filename, "rb");
+    ctx->file = xps_fopen(ctx->memory, filename, "rb");
     if (!ctx->file)
         return gs_throw1(-1, "cannot open file: '%s'", filename);
 
@@ -499,7 +782,7 @@ xps_process_file(xps_context_t *ctx, char *filename)
         int size;
 
         if_debug0m('|', ctx->memory, "zip: single page mode\n");
-        xps_strlcpy(buf, filename, sizeof buf);
+        gs_strlcpy(buf, filename, sizeof buf);
         while (1)
         {
             p = strrchr(buf, '/');
@@ -507,9 +790,9 @@ xps_process_file(xps_context_t *ctx, char *filename)
                 p = strrchr(buf, '\\');
             if (!p)
                 break;
-            xps_strlcpy(p, "/_rels/.rels", buf + sizeof buf - p);
+            gs_strlcpy(p, "/_rels/.rels", buf + sizeof buf - p);
             if_debug1m('|', ctx->memory, "zip: testing if '%s' exists\n", buf);
-            if (isfile(buf))
+            if (isfile(ctx->memory, buf))
             {
                 *p = 0;
                 ctx->directory = xps_strdup(ctx, buf);
@@ -524,11 +807,29 @@ xps_process_file(xps_context_t *ctx, char *filename)
             ctx->directory = xps_strdup(ctx, "");
         }
 
-        fseek(ctx->file, 0, SEEK_END);
-        size = ftell(ctx->file);
-        fseek(ctx->file, 0, SEEK_SET);
+        if (xps_fseek(ctx->file, 0, SEEK_END) != 0) {
+            code = gs_rethrow(gs_error_ioerror, "xps_fseek to file end failed");
+            goto cleanup;
+        }
+
+        size = xps_ftell(ctx->file);
+        if (size < 0) {
+            code = gs_rethrow(gs_error_ioerror, "xps_ftell raised an error");
+            goto cleanup;
+        }
+
+        if (xps_fseek(ctx->file, 0, SEEK_SET) != 0) {
+            code = gs_rethrow(gs_error_ioerror, "xps_fseek to file begin failed");
+            goto cleanup;
+        }
+
         part = xps_new_part(ctx, filename, size);
-        fread(part->data, 1, size, ctx->file);
+        code = xps_fread(part->data, 1, size, ctx->file);
+        if (code != size) {
+            code = gs_rethrow1(gs_error_ioerror, "failed to read %d bytes", size);
+            xps_free_part(ctx, part);
+            goto cleanup;
+        }
 
         code = xps_parse_fixed_page(ctx, part);
         if (code)
@@ -545,7 +846,7 @@ xps_process_file(xps_context_t *ctx, char *filename)
 
     if (strstr(filename, "/_rels/.rels") || strstr(filename, "\\_rels\\.rels"))
     {
-        xps_strlcpy(buf, filename, sizeof buf);
+        gs_strlcpy(buf, filename, sizeof buf);
         p = strstr(buf, "/_rels/.rels");
         if (!p)
             p = strstr(buf, "\\_rels\\.rels");
@@ -593,6 +894,17 @@ xps_process_file(xps_context_t *ctx, char *filename)
         }
     }
 
+    /* If we have a page list, adjust pages now */
+    if (ctx->page_range && ctx->page_range->page_list)
+    {
+        code = xps_reorder_pages(ctx);
+        if (code)
+        {
+            code = gs_rethrow(code, "invalid page range setting");
+            goto cleanup;
+        }
+    }
+
     for (page = ctx->first_page; page; page = page->next)
     {
         code = xps_read_and_process_page_part(ctx, page->name);
@@ -609,6 +921,6 @@ cleanup:
     if (ctx->directory)
         xps_free(ctx, ctx->directory);
     if (ctx->file)
-        fclose(ctx->file);
+        xps_fclose(ctx->file);
     return code;
 }
